/**
 * This package is DOMLET!!!
 */
package net.sf.jolene.dom;

import net.sf.jolene.constants.Prefs;
import net.sf.jolene.constants.Tags;
import net.sf.jolene.html.Attributes;
import net.sf.jolene.html.IAttributes;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.*;
import java.util.*;

/**
 * HTML Document object.
 *
 * @author Dan Howard
 * @since Sep 21, 2003 5:22:33 PM
 */
public final class Document {

    // todo modifiy file constructor to use File object - add constructor passing in the content of the document

    private static final Logger log = LogManager.getLogger(Document.class);

    // These are default protection so they are available to the Parser. The Parser's job is
    // to initialize and define these objects.
    String title;
    List<Header> head;
    String doctype;     // Store the text before the start of HTML - usually !DOCTYPE
    String bodyText;    // Body text string.

    List<Form> forms;
    IAttributes body;

    private String fileName;
    private String contextPath;
    private String uri;

    /**
     * Indicator for labels being translated.  We do not do the translations here.
     * We expect that the translation is done at the application level. This can
     * therefore be used to decide if the document is already translated (so you
     * don't need to do it over again.
     * <p/>
     * see net.sf.jolene.struts.DomletAction for the struts implementation
     */
    private boolean translated;

    /**
     * @param fileName
     * @param contextPath
     * @param uri
     * @throws IOException
     * @throws MalformedHTMLException - runtime exception
     */
    public Document(String fileName, String contextPath, String uri) throws IOException, MalformedHTMLException {
        this(fileName);
        this.contextPath = contextPath;
        this.uri = uri;
        translatePaths();
    }

    /**
     * @param fileName - file name of the document.
     * @throws IOException            - if an IOException occurs
     * @throws MalformedHTMLException - runtime exception
     */
    public Document(String fileName) throws IOException, MalformedHTMLException {
        this();
        this.fileName = fileName;
        open();
    }

    /**
     * For clone only.
     */
    Document() {
        title = "";
        head = new ArrayList<Header>(4);
        doctype = "";
        bodyText = "";
        contextPath = "";
        uri = "";
        body = new Attributes();
        forms = new ArrayList<Form>(4);
        uri = "";
    }


    /**
     * Adds a header element to the document.
     *
     * @param header a Header object.
     * @return boolean to indicate success.
     */
    public boolean addHeader(Header header) {
        return head.add(header);
    }

    /**
     * This is a custom clone implementation which ensures the document
     * is cleanly and safely cloned. Used by the DocumentFactory.
     *
     * @return Document
     */
    @Override
    public Document clone() {
        Document doc = new Document();
        doc.bodyText = bodyText;
        doc.fileName = fileName;
        doc.uri = uri;
        doc.doctype = doctype;
        doc.title = title;

        Iterator<Header> ith;
        ith = head.iterator();
        while (ith.hasNext()) {
            doc.head.add(ith.next().clone());
        }

        Iterator it;
        doc.body = new Attributes();
        it = body.keySet().iterator();
        while (it.hasNext()) {
            Object key;
            Object value;
            key = it.next();
            value = body.getAttribute(key.toString());
            doc.body.setAttribute(key.toString(), value.toString());
        }

        doc.forms = new ArrayList<Form>(forms.size());
        for (int i = 0; i < forms.size(); i++) {
            doc.forms.add(forms.get(i).clone());
        }

        return doc;
    }

    /**
     * Returns a object by searching all forms in the document.
     *
     * @param id The id of the element.
     * @return HTMLElement
     */
    public HTMLElement elements(String id) {
        HTMLElement element = null;

        if (forms != null && forms.size() > 0) {
            for (int j = 0; j < forms.size(); j++) {
                element = forms.get(j).elements(id);
                if (element != null) {
                    return element;
                }
            }
        }

        return element;
    }

    /**
     * Returns the form object specified by the number.
     *
     * @param index form number (starting with 0)
     * @return Form a Form object
     * @throws IndexOutOfBoundsException - if the index is out of range (index < 0 || index >= size()).
     */
    public Form forms(int index) {
        return forms.get(index);
    }

    /**
     * Returns a form object specified by name. For the form to be found it will require a 'name' attribute.
     * @param name name of the form in the document (case insensitive).
     * @return Form a Form object or null if not found.
     */
    public Form forms(String name) {
        for (Form form: forms) {
            if (name.equalsIgnoreCase(form.getName())) {
                return form;
            }
        }
        return null;
    }

    /**
     * Retrives the body attributes.
     *
     * @return IAttributes
     */
    public IAttributes getBody() {
        return body;
    }

    /**
     * Gets the doc type of the document.
     *
     * @return String
     */
    public String getDoctype() {
        return doctype;
    }


    /**
     * @return Number of forms in the document.
     */
    public int getFormCount() {
        return forms.size();
    }

    /**
     * Returns the number of Header elements in the document.
     *
     * @return int number of Header elements.
     */
    public int getHeaderCount() {
        return head.size();
    }

    /**
     * Gets the document title.
     *
     * @return title
     */
    public String getTitle() {
        return title;
    }

    /**
     * @return uri string for this document.
     */
    public String getUri() {
        return uri;
    }

    /**
     * Return the Header element specified by the index based on document order.
     *
     * @param header to the Header (starting with 0).
     * @return Header element.
     */
    public Header headers(int header) {
        return head.get(header);
    }

    /**
     * Indicator for labels being translated.  We do not do the translations here.
     * We expect that the translation is done at the application level. This can
     * therefore be used to decide if the document is already translated (so you
     * don't need to do it over again.
     * <p/>
     * see net.sf.jolene.struts.DomletAction for the struts implementation
     *
     * @return true if this document is translated
     */
    public boolean isTranslated() {
        return translated;
    }

    /**
     * Remove the headers
     * @param index to the header in the document. Zero based in document order.
     * @return The removed header object.
     */
    public Header removeHeader(int index) {
        return head.remove(index);
    }

    /**
     * Sets the doc type of the document.
     *
     * @param doctype The HTML doctype string.
     */
    public void setDoctype(String doctype) {
        this.doctype = doctype;
    }

    /**
     * Set the document title.
     *
     * @param string The document title.
     */
    public void setTitle(String string) {
        title = string;
    }

    /**
     * Indicator for labels being translated.  We do not do the translations here.
     * We expect that the translation is done at the application level. This can
     * therefore be used to decide if the document is already translated (so you
     * don't need to do it over again.
     * see net.sf.jolene.struts.DomletAction for the struts implementation
     *
     * @param translated set to true if the doc has been translated.
     */
    public void setTranslated(boolean translated) {
        this.translated = translated;
    }

    /**
     * Writes the document to the specified fileName.
     *
     * @param filename The filename to write to.
     * @throws java.io.IOException if there's an IOException
     */
    public void stream(String filename) throws IOException {
        BufferedWriter out = null;
        try {
            out = new BufferedWriter(new FileWriter(filename));
            stream(out);
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                }
            }
        }
    }

    /**
     * Writes the document to the specified Writer object and closes the Writer.
     *
     * @param out an output Writer to write to.
     * @throws java.io.IOException it there's an IOException
     */
    public void stream(Writer out) throws IOException {
        try {
            String ls = System.getProperty("line.separator");

            if (doctype != null && doctype.trim().length() > 0) {
                out.write(doctype + ls);
            }
            out.write("<html>" + ls);
            out.write("<head>" + ls);
            for (Header aHead : head) {
                out.write(aHead.getBeforeText() + aHead + aHead.getAfterText() + ls);
            }
            out.write("<title>" + getTitle() + "</title>" + ls);
            out.write("</head>" + ls);

            out.write("<body ");
            for (String s : body.keySet()) {
                out.write(s + "=\"" + body.getAttribute(s) + "\" ");
            }
            out.write(">" + ls);

            // For each form, put the text from the start of the body text to
            // the startpoint of the form.
            int offset;
            offset = 0;
            Form form;

            for (int j = 0; j < forms.size(); j++) {
                form = (Form) forms(j);
                // Write whatever text exist between previous form (or start of doc) and current form.
                out.write(bodyText.substring(offset, form.getStartPoint()));
                out.write(form.getBeforeText() + form + form.getAfterText());
                offset = form.getEndPoint() + 1;

                for (int k = 0; k < form.elements.size(); k++) {
                    //System.out.print("k = " + k + " " + form.elements(k).startPoint + " " + form.elements(k).endPoint + " " + offset);
                    //System.out.println(" ID: " + form.elements(k).getAttribute("id"));

                    out.write(bodyText.substring(offset, form.elements(k).getStartPoint()));
                    out.write(form.elements(k).getBeforeText() + form.elements(k) + form.elements(k).getAfterText());
                    offset = form.elements(k).getEndPoint() + 1;
                }

                // if this is the last form write the remaining text.
                if (j == forms.size() - 1) {
                    out.write(bodyText.substring(offset, bodyText.length()));
                }
            }

            out.write(ls);
            out.write("</body>" + ls);
            out.write("</html>");
            out.flush();
        } finally {
            try {
                out.close();
            } catch (IOException e) {
            }
        }
    }

    /**
     * Returns the document file name.
     *
     * @return String
     */
    public String toString() {
        return fileName;
    }

    //================================================

    void open() throws IOException, MalformedHTMLException {
        long starttime = System.currentTimeMillis();
        long fileSize = new File(fileName).length();

        log.debug("FileName:" + fileName + " fileSize: " + fileSize);
        BufferedReader reader = new BufferedReader(new FileReader(fileName));
        char[] buff = new char[(int) fileSize];

        String fileBuffer = "";
        try {
            int n = reader.read(buff, 0, (int) fileSize);

            // Convert char array to StringBuffer
            fileBuffer = new String(buff, 0, n);
        }

        finally {
            try {
                reader.close();
            } catch (IOException e) {
            }
        }

        String workBuffer = new String(fileBuffer);

        new Parser(workBuffer, this);

        if ("true".equalsIgnoreCase(Prefs.GenerateLocaleStrings.getValue())) {
            generateLocaleStrings();
        }

        log.info("Time to open doc: " + (System.currentTimeMillis() - starttime));
    }

    private void generateLocaleStrings() throws IOException {
        Properties properties = new Properties();
//@todo - so many issues here. Null pointer possibility, What value for objects? Which objects? Should they include html? NO! Wrong place for the code. Where to create the file?

        FileOutputStream out = null;
        FileInputStream in = null;
        try {
            File f = new File("resources/application.properties");
            f.createNewFile();
            in = new FileInputStream(f);
            properties.load(in);
            out = new FileOutputStream(f);

            Form ff;
            StringBuilder objectName;

            log.info("FILENAME: " + this.fileName);
            String docName;
            docName = this.fileName.substring(this.fileName.lastIndexOf("/") + 1);
            docName = docName.substring(0, docName.indexOf("."));
            log.info("DOC NAME: " + docName);
            for (int j = 0; j < forms.size(); j++) {
                objectName = new StringBuilder();
                objectName.append(docName).append('.');

                ff = (Form) forms(j);
                if (ff.getName().trim().length() > 0) {
                    objectName.append(ff.getName()).append('.');
                } else {
                    objectName.append("form").append(j + 1).append('.');
                }

                String elementName;
                HTMLElement element;
                for (int k = 0; k < ff.elements.size(); k++) {
                    element = ff.elements(k);
                    if (element.hasAttribute("name")) {
                        elementName = element.getAttribute("name");
                    } else if (element.hasAttribute("id")) {
                        elementName = element.getAttribute("id");
                    } else {
                        elementName = element.getTag().toString() + k;
                    }
                    if (element.getValue() != null) {
                        properties.put(objectName + elementName, element.getValue());
                    }
                }
            }
            properties.store(out, "---Locale strings written by jolene---");
            out.close();
            in.close();
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    log.warn(e.getMessage(), e);
                }
                try {
                    in.close();
                } catch (IOException e) {
                    log.warn(e.getMessage(), e);
                }
            }
        }
    }

    //--------------------------------------------------------

    //@TODO Add call for JUnit tests

    void streamObjects() {
        Form ff;

        for (int j = 0; j < forms.size(); j++) {
            ff = (Form) forms(j);

            System.out.println("FORM: " + j + ff);
            for (int k = 0; k < ff.elements.size(); k++) {
                System.out.print("" + ff.elements(k));
                if (ff.elements(k).hasAttribute("name")) {
                    System.out.println("NAME: " + ff.elements(k).getAttribute("name"));
                } else {
                    if (ff.elements(k).hasAttribute("id")) {
                        System.out.println("ID: " + ff.elements(k).getAttribute("ID"));
                    } else {
                        System.out.println();
                    }
                }
            }
        }
    }

    /**
     * For JUnit
     */
    void streamResults(Writer writer) throws IOException {
        for (int j = 0; j < forms.size(); j++) {
            writer.write("form : " + j + " " + forms(j) + "\n");
            for (int k = 0; k < forms(j).elements.size(); k++) {
                writer.write("element : " + k + "\n" + forms(j).elements(k) + "\n");
            }
        }
    }

    private void translatePaths() {
        HTMLElement element;
        String src;

        for (int j = 0; j < this.forms.size(); j++) {
            for (int k = 0; k < this.forms.get(j).getElementCount(); k++) {
                element = this.forms.get(j).elements(k);
                if (element.getTag().equals(Tags.img) && element.hasAttribute("src")) {
                    src = translatePath(element.getAttribute("src"));
                    element.setAttribute("src", src);
                }
            }
        }

        for (int j = 0; j < this.head.size(); j++) {
            element = this.head.get(j);

            if (element.tag == Tags.script && element.hasAttribute("src")) {
                src = translatePath(element.getAttribute("src"));
                element.setAttribute("src", src);
            } else if (element.tag == Tags.link) {
                if (element.hasAttribute("href")) {
                    src = translatePath(element.getAttribute("href"));
                    element.setAttribute("href", src);
                }

                if (element.hasAttribute("src")) {
                    src = translatePath(element.getAttribute("srg"));
                    element.setAttribute("src", src);
                }
            }
        }
    }

    private String translatePath(String path) {
        if (path == null) {
            // If an element does not have a src or href attribute then it will be assigned blank
            return "";
        }
        if (path.toLowerCase().startsWith("http")) {
            // leave it alone, the path exists on some other server.
            return path;
        }
        path = path.replace('\\', '/');
        StringTokenizer st = new StringTokenizer(path, "/");
        StringBuffer endPath = new StringBuffer("");
        String pathElement;
        while (st.hasMoreElements()) {
            pathElement = st.nextElement().toString();
            if (pathElement.equals(".") || pathElement.equals("..")) {
            } else {
                endPath.append('/').append(pathElement);
            }
        }

        path = contextPath + endPath;

        return path;
    }
}