package net.sf.jolene.dom;

import net.sf.jolene.constants.StyleClasses;
import net.sf.jolene.constants.Tags;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A renderable data bound grid object. A grid is detected by the parser by having a table using an ID tag
 * that starts with 'grid'.  The grid object can be accessed in the document just like any other object.
 * See the bind method to see how to 'datalink' the grid to a list of data objects.
 *
 * @author Dan Howard
 * @since Aug 1, 2005 5:54:59 PM
 */
public final class Grid extends HTMLElement {

    private static final Logger log = LogManager.getLogger(Grid.class);

    /**
     * List of row objects created when a document is read.
     */
    List<GridRow> rows;

    /**
     * Default constructor.
     */
    public Grid() {
        tag = Tags.table;
        rows = new ArrayList<GridRow>(0);
    }

    /**
     * Binds the grid object to the specified data.
     *
     * @param header List of GridColumn objects defining the column information for the grid.
     * @param fields List of fields (property names) which are accessed via PropertyUtils to set the values for each cell.
     * @param data   List of data objects used to retrieve the values.
     * @throws GridBindException if PropertyUtils fails.
     */
    public void bind(List<GridColumn> header, List<String> fields, List data) throws GridBindException {
        rows = new ArrayList<GridRow>(data.size());

        if (header.size() > 0) {
            Iterator<GridColumn> it = header.iterator();
            rows.add(new GridRow());
            rows.get(0).setHeader(true);
            while (it.hasNext()) {
                rows.get(0).cells.add(new GridCell(it.next()));
            }
        }

        if (data == null || data.size() == 0) {
            return;
        }

        int rowPosition = 1;
        Iterator it = data.iterator();

        while (it.hasNext()) {
            Object row;
            row = it.next();
            rows.add(new GridRow());

            Iterator<String> fit = fields.iterator();

            int colPosition = 0;
            while (fit.hasNext()) {
                String fieldName = fit.next();

                // Here we get the associated grid column header since it might
                // have width and align properties defined which need be set on
                // each cell.
                GridColumn col = header.get(colPosition);
                GridCell cell;
                cell = new GridCell(col);
                if (col.getCellObject() != null) {
                    cell.setCellObject(col.getCellObject().clone());

                    int n1;
                    String cellName = cell.getCellObject().getName();
                    n1 = cellName.indexOf("${");
                    if (n1 > -1) {
                        // Means the user has a indexed property needs to be in a format propertyName(${indexerFieldName})
                        // parse out the field name.
                        int n2 = cellName.indexOf("}");
                        String propertyName = cellName.substring(n1 + 2, n2).trim();

                        try {
                            cellName = cellName.substring(0, n1) + PropertyUtils.getProperty(row, propertyName) + cellName.substring(n2 + 1);
                        } catch (Exception e) {
                            throw new GridBindException(e);
                        }
                        cell.getCellObject().setName(cellName);
                    }
                }

                log.debug(fieldName + " " + row.getClass().getName());
                try {
                    cell.setValue("" + PropertyUtils.getProperty(row, fieldName));
                } catch (Exception e) {
                    throw new GridBindException(e);
                }
                rows.get(rowPosition).cells.add(cell);

                if (rowPosition % 2 == 0) {
                    rows.get(rowPosition).setAttribute("class", StyleClasses.EvenRow.toString());
                } else {
                    rows.get(rowPosition).setAttribute("class", StyleClasses.OddRow.toString());
                }
                colPosition++;
            }
            rowPosition++;
        }
    }


    /**
     * Returns a clone of the grid object.
     *
     * @return Grid object.
     */
    @Override
    public Grid clone() {
        Grid grid = (Grid) super.clone();

        for (int j = 0; j < rows.size(); j++) {
            GridRow r = rows(j).clone();
            for (int i = 0; i < rows(j).cells.size(); i++) {
                r.cells.add(rows(j).cells(i).clone());
            }
            grid.rows.add(r);
        }
        return grid;
    }

    /**
     * Renders the grid.
     *
     * @return Grid as html string.
     */
    @Override
    public String toString() {

        if (swapWith != null) {
            return swapWith.toString();
        }

        setAttribute("class", StyleClasses.Grid.toString());

        StringBuilder sb = new StringBuilder(super.toString());
        String lf = System.getProperty("line.separator");

        sb.append(lf);

        Iterator it = rows.iterator();
        while (it.hasNext()) {
            sb.append(it.next().toString()).append(lf);
        }
        sb.append("</").append(tag).append(">").append(lf);
        return sb.toString();
    }

    /**
     * @param row row index to return.
     * @return specified GridRow.
     */
    GridRow rows(int row) {
        return rows.get(row);
    }
}
