package net.sf.jolene.dom;

import net.sf.jolene.constants.Prefs;
import net.sf.jolene.constants.Tags;
import net.sf.jolene.html.Attributes;
import net.sf.jolene.html.IAttributes;
import net.sf.jolene.html.IStyles;
import net.sf.jolene.html.Styles;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * Base class for all html elements.
 *
 * @author Dan Howard
 * @since Oct 11, 2003
 */
public abstract class HTMLElement implements IAttributes, IStyles, Cloneable {
// ------------------------------ FIELDS ------------------------------

    // todo implement default attriutes for objects
    static final Logger log = LogManager.getLogger(HTMLElement.class);

    // This will contain the starting chars when streaming. e.g.
    // INPUT TYPE = 'INPUT'
    // SELECT     = 'SELECT'
    // LABEL      = 'LABEL'
    Tags tag;

    // For swapping
    String swapWith;

    // Custom properties
    private String afterText;
    private String beforeText;

    // These store the location in the file - internal
    private int startPoint;
    private int endPoint;

    // Common properties.
    private String name;
    private String value;
    private String content; // inner content of an element between ending of start tag and end tag. http://www.w3.org/TR/html401/interact/forms.html#h-17.5
    private boolean renderable;

    // HTML attributes - has access only with package private setters/getters.
    private IAttributes attributes;

    private IStyles styles;

    protected HTMLElement() {
        afterText = "";
        beforeText = "";
        renderable = true;
        name = "";
        value = "";
        content = "";
        attributes = new Attributes();
        styles = new Styles();
        tag = Tags.input;
        startPoint = 0;
        endPoint = 0;
    }

    protected HTMLElement(String name) {
        this();
        this.name = name;
    }

    /**
     * Clears this element's attributes.
     */
    public void clear() {
        attributes.clear();
    }

    /**
     * Core clone implementation. Safely clones HTMLElement objects. Used by the DocumentFactory.
     *
     * @return HTMLElement
     * @see net.sf.jolene.factories.DocumentFactory
     */
    @Override
    public HTMLElement clone() {
        HTMLElement element;
        boolean setValue = true;

        if (this instanceof Button) {
            element = new Button();
        } else if (this instanceof CheckBox) {
            element = new CheckBox();
        } else if (this instanceof Grid) {
            element = new Grid();
            setValue = false;
        } else if (this instanceof Image) {
            element = new Image();
            setValue = false;
        } else if (this instanceof Input) {
            element = new Input();
        } else if (this instanceof Label) {
            element = new Label();
        } else if (this instanceof Radio) {
            element = new Radio();
        } else if (this instanceof Select) {
            element = new Select();
        } else if (this instanceof TextArea) {
            element = new TextArea();
        } else if (this instanceof GridCell) {
            element = new GridCell();
        } else if (this instanceof GridRow) {
            element = new GridRow();
        } else if (this instanceof Header) {
            element = new Header();
        } else {
            element = new Form();
            setValue = false;
        }

        // Standard clone
        Iterator<String> it;
        String key;
        String value;
        it = attributes.keySet().iterator();
        while (it.hasNext()) {
            key = it.next();
            value = getAttribute(key);
            element.setAttribute(key, value);

            // Make sure to call setValue
            if ("value".equalsIgnoreCase(key)) {
                element.setValue(value);
            }
        }
        // Use setters instead of direct access in case we override these methods (setName is overriden for example).
        element.setName(name);
        element.setAfterText(afterText);
        element.setBeforeText(beforeText);
        element.setEndPoint(endPoint);
        element.setRenderable(renderable);
        element.setStartPoint(startPoint);
        element.setTag(tag);
        element.swapWith(swapWith);

        // Set the value ONLY for objects that have a value
        // Note that the object could have a null in the case where the object
        // usually has a value attribute in HTML but was not specified in the
        // actual document. example:
        // <INPUT NAME="username" TYPE="text">
        //@todo why do we bother checking if element.getValue is null? It means that we always need a default for GridCell, TextArea, etc (ones which override the default get/set value methods)
        if (setValue && this.value.trim().length() > 0) {
            element.setValue(this.value);
        }

        return element;
    }

    /**
     * Gets the afterText string for the object.
     * The afterText is any string value that you want to be rendered immediatley after the object itself.
     *
     * @return String
     */
    public String getAfterText() {
        return afterText;
    }

    /**
     * Return the html attribute specified by the key or null if it's not found.
     *
     * @param key case insensitive attribute name.
     * @return attribute value string.
     */
    public final String getAttribute(String key) {
        return attributes.getAttribute(key);
    }

    /**
     * Gets the beforeText string for the object.
     * The beforeText is any string value that you want to be rendered immediatley before
     * the object itself.
     *
     * @return String
     */
    public String getBeforeText() {
        return beforeText;
    }

    /**
     * <p/>
     * Returns the content of the HTMLElement. The content is part of the element between the end of the start tag
     * and the beginning of the end tag. In the example below <b>Button text</b> is the content.
     * </p>
     * <tt>
     * &lt;button name="test"&gt;Button text&lt;/button&gt;
     * </tt>
     * <p/>
     * Note that some elements do not have content. For these elements this content
     * property has no effect on the rendering of the element.
     * </p>
     * <p/>
     * Elements which have a content.
     * <ul>
     * <li>Button</li>
     * <li>Label</li>
     * <li>TextArea</li>
     * <li>Header <i>Some header elements do and some don't.</i></li>
     * </ul>
     * <p/>
     * <br>
     * Elements which do NOT have a content.
     * <ul>
     * <li>ChecBox</li>
     * <li>Grid</li>
     * <li>Image</li>
     * <li>Input</li>
     * <li>Radio</li>
     * <li>Select</li>
     * </ul>
     * <p/>
     * <p/>
     * Note also that for TextArea, Text and Label the content is also the value. In this case the value and content properties are
     * interchangable.
     * </p>
     *
     * @return String content part of the HTMLElement
     */
    public String getContent() {
        return content;
    }

    /**
     * Gets the element name.
     *
     * @return name of the html element.
     */
    public String getName() {
        return name;
    }


    /**
     * Get the element style based on a style name.
     *
     * @param key case insensitive style name.
     * @return String value for the specified style or null if not found.
     */
    public String getStyle(String key) {
        return styles.getStyle(key);
    }

    /**
     * Get the tag for the element.
     *
     * @return the tag for the element.
     */
    public Tags getTag() {
        return tag;
    }

    /**
     * Gets the current value of the element.
     *
     * @return String
     */
    public String getValue() {
        // Default for input type text, buttons etc....
        //return getAttribute("value");
        return value;
    }

    /**
     * Indicates if the specified attribute exists.
     *
     * @param key case insensitive attribute name.
     * @return true if the attribute exists.
     */
    public boolean hasAttribute(String key) {
        return attributes.hasAttribute(key);
    }

    /**
     * Indicates if a style exists.
     *
     * @param key case instensitive style key name.
     * @return true if the style exists in the string.
     */
    public boolean hasStyle(String key) {
        return styles.hasStyle(key);
    }


    /**
     * Returns true if a Radio or CheckBox is checked.
     *
     * @return checked
     */
    public boolean isChecked() {
        return hasAttribute("checked");
    }

    /**
     * Returns true if the element is disabled
     *
     * @return disabled
     */
    public boolean isDisabled() {
        return hasAttribute("disabled");
    }


    /**
     * Returns true if the element is readonly
     *
     * @return readonly
     */
    public boolean isReadonly() {
        return hasAttribute("readonly");
    }

    /**
     * Returns whether this element is rendered. If this is false then the element will be rendered as an empty string.
     *
     * @return boolean
     */
    public boolean isRenderable() {
        return renderable;
    }


    /**
     * Returns the set of attribute names.
     *
     * @return set of attribute names.
     */
    public Set<String> keySet() {
        return attributes.keySet();
    }

    /**
     * Remove the specified attribute.
     *
     * @param key case insensitive attribute name.
     * @return previous value associated with specified key, or <tt>null</tt> if there was no mapping for key.
     */
    public String removeAttribute(String key) {
        return attributes.removeAttribute(key);
    }

    /**
     * Removes a style from the element.
     *
     * @param key Style key name to remove.
     * @return previous value associated with specified key, or <tt>null</tt> if there was no mapping for key.
     */
    public String removeStyle(String key) {
        return styles.removeStyle(key);
    }

    /**
     * Sets the styles on the element based on the formatted style string, clearing all exisiting styles first.
     *
     * @param style string in a format like <tt>azimuth:behind;background:aliceblue;background-color:aquamarine;border-bottom-style:solid;clip:auto;border-top-width:medium;</tt>
     */
    public final void resetStyles(String style) {
        styles.resetStyles(style);
        setAttribute("style", style);
    }

    /**
     * Sets the styles on the element based on the Style parameter, clearing all exisiting styles first.
     *
     * @param style a Style object.
     */
    public final void resetStyles(Styles style) {
        resetStyles(style.toString());
    }

    /**
     * Sets the afterText string for the object.
     * The afterText is any string value that you want to be rendered immediately after the object itself.
     * <p/>
     * Example:
     * <pre>
     * document.forms(0).elements("Submit").setAfterText(" Click this button to submit this form.");
     * </pre>
     * This would render the following back to the browser:
     * <pre>
     * &lt;input type="SUBMIT" name="Submit" value="Submit"&gt; Click this button to submit this form.
     * </pre>
     *
     * @param string text after object
     */
    public void setAfterText(String string) {
        afterText = string;
    }

    /**
     * Sets the specified attribute.
     *
     * @param key   case insensitive attribute name.
     * @param value value of the attribute.
     * @return previous value associated with specified key, or <tt>null</tt> if there was no attribute for the key.
     */
    public final String setAttribute(String key, String value) {
        if (value == null) {
            value = "";
        }

        if ("style".equalsIgnoreCase(key)) {
            styles.setStyles(value);
            String style = styles.toString();
            attributes.setAttribute(key, style);
            return style;
        }

        return attributes.setAttribute(key, value);
    }

    /**
     * Sets the beforeText string for the object.
     * The beforeText is any string value that you want to be rendered immediatley before
     * the object itself.
     * <p/>
     * Example:
     * <pre>
     * document.forms(0).elements("Name").setBeforeText("Name: ");
     * </pre>
     * This would render the following back to the browser:
     * <pre>
     * Name: &lt;input type="text" name="Name"&gt;
     * </pre>
     *
     * @param string of text
     */
    public void setBeforeText(String string) {
        beforeText = string;
    }

    /**
     * Sets the element checked attribute. Used for Radios and Checkboxes.
     *
     * @param checked to indicate if the checkbox is checked or not.
     */
    public void setChecked(boolean checked) {
        if (checked) {
            setAttribute("checked", "checked");
        } else {
            if (hasAttribute("checked")) {
                removeAttribute("checked");
            }
        }
    }

    /**
     * Sets the content of the HTMLElement.
     *
     * @param content text or value to set.
     * @see HTMLElement#getContent()
     */
    public void setContent(String content) {
        this.content = content;
    }

    /**
     * Set the element to be disabled.
     *
     * @param disabled indicator for a disabled element.
     */
    public void setDisabled(boolean disabled) {
        if (disabled) {
            setAttribute("disabled", "disabled");
        } else {
            if (hasAttribute("disabled")) {
                removeAttribute("disabled");
            }
        }
    }

    /**
     * Set the element name.
     *
     * @param name name of the html element.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Set the element to be readonly.
     *
     * @param readonly indicator for a readonly element.
     */
    public void setReadonly(boolean readonly) {
        if (readonly) {
            setAttribute("readonly", "readonly");
        } else {
            if (hasAttribute("readonly")) {
                removeAttribute("readonly");
            }
        }
    }


    /**
     * Sets whether this element is rendered. If this is false then the element will be rendered as an empty string.
     *
     * @param renderable true or false
     */
    public void setRenderable(boolean renderable) {
        this.renderable = renderable;
    }

    /**
     * Set the element style.
     *
     * @param key   Style key name.
     * @param value Style value.
     * @return previous value associated with specified key, or <tt>null</tt> if there was no mapping for key.
     */
    public String setStyle(String key, String value) {
        return setAttribute("style", key + ":" + value + ";");
    }

    /**
     * Sets the styles on the element based on a formatted style string.
     * Expects the string to be in style format.
     *
     * @param style string in a format like <tt>azimuth:behind;background:aliceblue;background-color:aquamarine;border-bottom-style:solid;clip:auto;border-top-width:medium;</tt>
     */
    public final void setStyles(String style) {
        styles.setStyles(style);
        setAttribute("style", styles.toString());
    }

    /**
     * Sets styles on the Style object based on the Style parameter.
     *
     * @param styles a Style object.
     */
    public final void setStyles(Styles styles) {
        setStyles(styles.toString());
    }


    // Non-public
    void setTag(Tags tag) {
        this.tag = tag;
    }

    /**
     * Sets the value of the element.
     *
     * @param value value of object
     */
    public void setValue(String value) {
        // Note textarea, select, label override this.
        // Default for setValue, textarea, select, label should override.
        // setAttribute("value", value);
        this.value = value;
    }

    /**
     * Replace this element's string representation with the specified string.
     *
     * @param swapWith any string you wish to render instead of this object.
     */
    public void swapWith(String swapWith) {
        this.swapWith = swapWith;
    }

    /**
     * Renders the element as an html string.
     */
    @Override
    public String toString() {
        if (swapWith != null) {
            return swapWith;
        }

        if (!renderable && !log.isDebugEnabled()) {
            return "";
        }

        StringBuffer sb = new StringBuffer("");
        Iterator iterator = attributes.keySet().iterator();

        sb.append('<').append(tag);

        List<BooleanAttributes> booleanAttributes = Arrays.asList(BooleanAttributes.values());

        while (iterator.hasNext()) {
            String key = (String) iterator.next();
            String value = attributes.getAttribute(key);
            if (value == null) {
                log.warn("Value is null? setting to blank string");
                value = "";
            }
            if (booleanAttributes.contains(BooleanAttributes.fromString(key.toLowerCase()))) {
                sb.append(' ').append(key).append('=').append('\"').append(key.toLowerCase()).append('\"');
            } else {
                String quote = "\"";
                // If the key itself contains a " character
                // use the single ' quote.
                if (value.indexOf("\"") > -1) {
                    quote = "'";
                }

                sb.append(' ').append(key).append('=').append(quote).append(value).append(quote);
            }
        }

        if (iterator.hasNext()) {
            sb.append(' ');
        } else {
            if ("true".equalsIgnoreCase(Prefs.XHTMLStrict.getValue())) {
                //@todo  /> is buggy!
                sb.append(" />");
            } else {
                sb.append('>');
            }
        }
        if (log.isDebugEnabled()) {
            sb.append("Debug: ");
            sb.append("BeforeText: ").append(beforeText).append(' ');
            sb.append("AfterText: ").append(afterText).append(' ');
            sb.append("Start point: ").append(startPoint).append(' ');
            sb.append("End point: ").append(endPoint).append(' ');
            sb.append("Render? ").append(renderable).append(' ');
            sb.append("TAG: ").append(tag);
        }
        return new String(sb);
    }

/* not used or needed we have keyset....
    public Collection<String> values() {
        return attributes.values();
    }
*/

    // Not public
    IAttributes getAttributes() {
        return attributes;
    }

    int getEndPoint() {
        return endPoint;
    }

    int getStartPoint() {
        return startPoint;
    }

    // Not public
    IStyles getStyles() {
        return styles;
    }

    // Not public
    void setAttributes(IAttributes attributes) {
        this.attributes = attributes;
    }

    void setEndPoint(int endPoint) {
        this.endPoint = endPoint;
    }

    void setStartPoint(int startPoint) {
        this.startPoint = startPoint;
    }
}
