package net.sf.jolene.struts;

import net.sf.jolene.util.PrefsReader;
import net.sf.jolene.dom.Document;
import net.sf.jolene.dom.HTMLElement;
import net.sf.jolene.dom.Label;
import net.sf.jolene.dom.Text;
import net.sf.jolene.factories.DocumentFactory;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;
import org.apache.struts.util.MessageResources;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * Default action servlet for jolene.  Implements a modified DispatchAction which can match request uri string
 * to the method name of the action class. The primary purpose is to associate a uri to a document object.
 * There are several ways to do this see the method descriptions.
 *
 * @author Dan Howard
 * @since Sep 23, 2006 5:40:15 PM
 */
public class DomletAction extends DispatchAction {

    private static final Logger log = LogManager.getLogger(DomletAction.class);

    /**
     * Performs a basic dispatch of the request looking for the parameter from the struts mapping or by using
     * the struts action name itself.
     *
     * @param mapping  ActionMapping
     * @param form     ActionForm
     * @param request  HttpServletRequest
     * @param response HttpServletResponse
     * @return ActionForward
     * @throws Exception
     */
    @Override
    public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {

        log.info("DomletAction Initializing PrefsReader...");
        PrefsReader.init();

        // Identify the request parameter containing the method name
        log.debug("QUERY_STRING :" + request.getQueryString());

        // Identify the method name to be dispatched to.
        // Either it's the parameter or the path name itself minus .do etc.
        String parameter = mapping.getParameter();
        String name;
        if (parameter != null) {
            name = request.getParameter(parameter);
        } else {
            String path = mapping.getPath();
            name = path;
            if (path.lastIndexOf("/") != -1) {
                name = path.substring(path.lastIndexOf("/") + 1);
            }
        }

        // Invoke the named method, and return the result
        return dispatchMethod(mapping, form, request, response, name);

    }


    /**
     * Retrives a document object based on the mapping input attribute and attaches it to the proper scope.
     * The reason for this method is to allow a default document to be associated with an action since
     * actions can have many forwards.
     * <p/>
     * For example, calling this method would get the document from the <code>/domlet/login.html<code> uri.
     * <p/>
     * <pre>
     * &lt;action
     * path="/login"
     * type="domlet.kbsample.web.actions.LoginAction"
     * name="LoginForm"
     * scope="session"
     * validate="false"
     * input="/domlet/login.html"&gt;
     * &nbsp;
     * &lt;forward name="success" path="/browse.do"/&gt;
     * &lt;forward name="fail" path="/domlet/error.html"/&gt;
     * &lt;forward name="login" path="/domlet/login.html"/&gt;
     * &lt;/action&gt;
     * </pre>
     *
     * @param mapping ActionMapping
     * @param request HttpServletRequest
     * @return Document or null if the uri for input mapping is undefined in struts-config.xml
     * @throws IOException if an IOException occurs
     */
    public Document getDocument(ActionMapping mapping, HttpServletRequest request) throws IOException {
        String uri = mapping.getInput();
        log.debug("mapping URI: " + uri);
        if (uri == null) {
            log.error("NO URI Specified in struts-config.xml for " + mapping.getPath());
            return null;
        }
        return getDocument(request, uri, mapping);
    }

    /**
     * Retrives a document object based on the specified forward.
     * <p/>
     * For example, using the following in struts-config.xml:
     * <p/>
     * <pre>
     * &lt;action
     * path="/login"
     * type="domlet.kbsample.web.actions.LoginAction"
     * name="LoginForm"
     * scope="session"
     * validate="false"
     * input="/domlet/login.html"&gt;
     * &nbsp;
     * &lt;forward name="success" path="/browse.do"/&gt;
     * &lt;forward name="fail" path="/domlet/error.html"/&gt;
     * &lt;forward name="login" path="/domlet/login.html"/&gt;
     * &lt;/action&gt;
     * </pre>
     * <p/>
     * <code>getDocument(mapping, "fail", request); </code> Returns document from /domlet/error.html <br>
     * <code>getDocument(mapping, "login", request);</code> Returns document from /domlet/login.html
     *
     * @param mapping ActionMapping
     * @param forward forward string
     * @param request HttpServletRequest
     * @return Document or null if the forward cannot be found in struts-config.xml
     * @throws IOException if an IOException occurs
     */
    public Document getDocument(ActionMapping mapping, String forward, HttpServletRequest request) throws IOException {

        String uri = mapping.findForward(forward).getPath();
        log.debug("Mapping URI from forward: " + forward + ": " + uri);
        return getDocument(request, uri, mapping);

    }

    private Document getDocument(HttpServletRequest request, String uri, ActionMapping mapping) throws IOException {
        String file;
        file = request.getSession().getServletContext().getRealPath(uri);

        Document document;
        if ("request".equalsIgnoreCase(mapping.getScope())) {
            if (request.getAttribute(uri) == null) {
                document = DocumentFactory.getInstance().getDocument(file, request.getContextPath(), uri);
            } else {
                document = (Document) request.getAttribute(uri);
            }
            request.setAttribute(uri, document);
        } else {
            if (request.getSession().getAttribute(uri) == null) {
                document = DocumentFactory.getInstance().getDocument(file, request.getContextPath(), uri);
            } else {
                document = (Document) request.getSession().getAttribute(uri);
            }
            request.getSession().setAttribute(uri, document);
        }

        translate(document, request);
        return document;
    }

    /**
     * Translates labels texts and input type button/submit
     *
     * @param document
     * @param request
     */
    private void translate(Document document, HttpServletRequest request) {
        if (document.isTranslated()) {
            return;
        }

        MessageResources messages = getResources(request);

        // Get the URI /somfolder/somehtml.html to convert to somefolder.somehtml as a start key for the resources
        String docKey = document.getUri();
        if (docKey.startsWith("/")) {
            docKey = docKey.substring(1);
        }
        int n = docKey.indexOf(".");
        if (n > -1) {
            docKey = docKey.substring(0, n);
        }
        docKey = docKey.replace("/", ".");
        docKey = docKey.replace("\\", ".");
        docKey = docKey.toLowerCase();
        //---------------------------------
        log.debug("docKey " + docKey);

        HTMLElement element;
        String formKey;
        for (int j = 0; j < document.getFormCount(); j++) {
            formKey = document.forms(j).getName();
            if (formKey.trim().length() == 0) {
                formKey = "form" + (j + 1);
            }

            String elementName;
            for (int k = 0; k < document.forms(j).getElementCount(); k++) {
                element = document.forms(j).elements(k);
                elementName = docKey + "." + formKey + "." + element.getName();
                log.debug("Element name " + elementName);

                // Check for labels.
                if (element instanceof Label && !(element instanceof Text)) {

                    if (messages.isPresent(elementName)) {
                        element.setValue(messages.getMessage(elementName));
                    }
                } else {
                    // Check for buttons - buttons display the value attribute
                    if (element.hasAttribute("type") && (element.getAttribute("type").equalsIgnoreCase("button") || element.getAttribute("type").equalsIgnoreCase("submit"))) {

                        if (messages.isPresent(elementName)) {
                            log.debug("setting element " + elementName);
                            element.setValue(messages.getMessage(elementName));
                        }
                    }
                }

                // check for title attribute
                if (element.hasAttribute("title")) {
                    log.debug("Element name title: " + elementName + ".title");
                    if (messages.isPresent(elementName + ".title")) {
                        log.debug("setting title " + elementName);
                        element.setAttribute("title", messages.getMessage(elementName + ".title"));
                    }
                }
            }
        }
        document.setTranslated(true);

    }


}
