package net.sf.jolene.dom;

import net.sf.jolene.constants.Elements;
import net.sf.jolene.constants.Tags;
import net.sf.jolene.html.Attributes;
import org.apache.commons.beanutils.BeanUtils;

import java.util.*;

/**
 * A form in a html document. This acts as a container for other html objects.
 *
 * @author Dan Howard
 * @since Oct 21, 2003
 */
public final class Form extends HTMLElement {

    List<HTMLElement> elements;

    Form() {
        tag = Tags.form;
        elements = new ArrayList<HTMLElement>(16);
    }


    @Override
    public Form clone() {
        Form form = (Form) super.clone();
        for (int j = 0; j < elements.size(); j++) {
            HTMLElement element = elements(j).clone();
            form.addElement(element);
        }
        return form;
    }


    /**
     * This method returns the widget specified by the numeric index.
     * The prefered way of retrieving an object from the form is to
     * use it's name or id instead since the order of the objects can be
     * changed easily by a page designer.
     *
     * @param element - The integer element number of the object desired.
     * @return HTMLElement
     */
    public HTMLElement elements(int element) {
        return elements.get(element);
    }

    /**
     * This method retrieves an object from the document by name or id.
     * When the document is opened it creates objects by looking at the ID
     * attribute first. If that attribute doesn't exist it uses the NAME
     * attribute. If neither exist then the only way to address these
     * objects is by number (not recommended).<br>
     * This method returns the base HTMLElement object so if you want a Select or
     * Grid object you can either cast it:
     * <pre>
     * Grid grid grid = (Grid)document.forms(0).elements("Grid1");
     * </pre>
     * or use the appropriate getter:
     * <pre>
     * Grid grid = document.forms(0).getGrid("Grid1");
     * </pre>
     *
     * @param elementName - The string name or id of the object desired.
     * @return HTMLElement or null if not found or elementName is null.
     */
    public HTMLElement elements(String elementName) {

        if (elementName == null) {
            return null;
        }

        // locate the object whose name == this string.
        String name = "";
        HTMLElement element;
        int j;

        for (j = 0; j < elements.size(); j++) {

            if (elementName.equalsIgnoreCase(elements.get(j).getName())) {
                break;
            }
/*
            if (element.hasAttribute("id")) {
                name = element.getAttribute("id");
                if (name.equalsIgnoreCase(elementName)) {
                    break; // found it.
                }
            } else if (element.hasAttribute("name")) {
                name = (String) element.getAttribute("name");
                if (name.equalsIgnoreCase(elementName)) {
                    break; // found it.
                }
            } else {
                // We have to throw an exception here? We have an object with no NAME!!!!!

            }
*/

        }

        if (j < elements.size()) {
            return elements(j);
        } else {
            // THROW EXCEPTION
            // Based on the HashMap, no exception is thrown but a null is returned which
            // will cause a null pointer exception later.
            return null;
        }
    }


    /**
     * @return Number of elements in the form.
     */
    public int getElementCount() {
        return elements.size();
    }

    /**
     * Returns the specified element casted to a Grid object.
     *
     * @param name name of grid
     * @return Grid
     */
    public Grid getGrid(String name) {
        return (Grid) elements(name);
    }

    /**
     * Returns a Map of Radio objects based the specified radio group name.
     * The map's order is insertion-order.
     *
     * @param groupName of radios
     * @return Map<String, HTMLElement>
     */
    public Map<String, HTMLElement> getRadioGroup(String groupName) {
        Map<String, HTMLElement> map = new LinkedHashMap<String, HTMLElement>(3);
        // loop through the objects for radios with the specified name
        // and add each to a hashmap based on their VALUE attribute.
        // If no value exists, log a warn and add each with an integer as
        // the key.
        for (int j = 0; j < elements.size(); j++) {
            HTMLElement element = elements.get(j);
            if (element instanceof Radio) {
                if (groupName.equalsIgnoreCase(element.getName())) {
                    log.debug("Radio " + groupName + " found.");
                    String key;
                    if (element.hasAttribute("value")) {
                        key = element.getAttribute("value");
                        log.debug("Radio " + groupName + " key " + key);
                    } else {
                        key = String.valueOf(j);
                        log.debug("getRadioGroup: Radio has no value: " + element + " assigning " + key);
                    }
                    map.put(key, element);
                }
            }
        }
        return map;
    }

    /**
     * Returns the specified element casted to a Select object.
     *
     * @param name name of select
     * @return Select
     */
    public Select getSelect(String name) {
        return (Select) elements(name);
    }

    /**
     * Checks if the specified object exists in the form.
     *
     * @param name name to check
     * @return boolean
     */
    public boolean hasElement(String name) {
        return elements(name) != null;
    }

/*
    public Label getLabel(String name) {
        return (Label) elements(name);
    }

    public Button getButton(String name) {
        return (Button) elements(name);
    }
*/

    /**
     * Poplates the form objects based on the supplied java bean.
     *
     * @param bean any object which can be examined by BeanUtils.
     * @throws FormPolulateException if BeanUtils fails.
     */
    public void populate(Object bean) throws FormPolulateException {

        Map<String, String> map = null;
        //@todo should this support nested properties like the grid?
        try {
            map = BeanUtils.describe(bean);
        } catch (Exception e) {
            throw new FormPolulateException(e);
        }
        populate(map);
    }

    /**
     * Populates the form object from the supplied map object.
     *
     * @param map - name/values where names match element names
     */
    public void populate(Map<String, String> map) {
        Map<String, String> radios = new HashMap<String, String>(3);

        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();

            // Prevent null values in the bean from throwing null pointer
            String value;
            if (map.get(key) == null) {
                value = "";
            } else {
                value = (String) map.get(key);
            }

            if (hasElement(key.toString())) {
                // Do not auto populate Radios since we don't really
                // know which radio to assign the value to since usually
                // they would have the same name. See getRadioGroup() method.
                if (elements(key) instanceof Radio) {
                    radios.put(key, "");
                } else {
                    if (elements(key) instanceof CheckBox) {
                        if (value.equalsIgnoreCase("true")) {
                            elements(key).setChecked(true);
                        } else {
                            elements(key).setChecked(false);
                        }
                    } else {
                        elements(key).setValue(value);
                    }
                }
            }
        }

        /*
        Check radios. If there are any radios that matched a field in the bean
        then it means that maybe that fields value matches a radio value from the
        group.  For each group check if the values match and set the CHECKED for
        the ones that do.
        */
        it = radios.keySet().iterator();
        while (it.hasNext()) {
            String key;
            key = it.next().toString();

            Iterator<HTMLElement> itGroup;
            itGroup = getRadioGroup(key).values().iterator();
            while (itGroup.hasNext()) {
                HTMLElement radio = itGroup.next();
                radio.setChecked(false);
                if (radio.getValue() != null && map.get(key) != null) {
                    if (radio.getValue().equalsIgnoreCase(map.get(key).toString())) {
                        radio.setChecked(true);
                    }
                }
            }
        }

    }

    final HTMLElement addObject(Elements type, Attributes attributes, int start, int end) {

        HTMLElement element;

        log.debug("TYPE:" + type);

        switch (type) {

            case select:
                element = new Select();
                break;

            case textarea:
                element = new TextArea();
                break;

            case label:
                element = new Label();
                break;

            case button:
                element = new Button();
                break;

            case image:
                element = new Image();
                break;

            case grid:
                element = new Grid();
                break;

            case radio:
                element = new Radio();
                break;

            case checkbox:
                element = new CheckBox();
                break;

            case text:
                element = new Text();
                break;

            default:
                element = new Input();
        }

        log.debug("CLASS:" + element.getClass().getName());

        Iterator i = attributes.keySet().iterator();

        while (i.hasNext()) {
            String key, value;
            key = (String) i.next();
            value = attributes.getAttribute(key);
            element.setAttribute(key, value);

            // Ensure to call setValue for values.
            if ("value".equalsIgnoreCase(key)) {
                element.setValue(value);
            }
        }

        // Also assign the object's name if found.
        if (attributes.hasAttribute("name")) {
            element.setName(attributes.getAttribute("name"));
        } else if (attributes.hasAttribute("id")) {
            element.setName(attributes.getAttribute("id"));
        }

        element.setStartPoint(start);
        element.setEndPoint(end);

        elements.add(element);
        return element;
    }

    /**
     * Used for cloning.
     *
     * @param element to add
     * @return HTMLElement
     */
    HTMLElement addElement(HTMLElement element) {
        elements.add(element);
        return element;
    }


}
