/*
 * Created on Dec 23, 2003
*/
package net.sf.jolene.dom;

import net.sf.jolene.constants.Prefs;
import net.sf.jolene.constants.Tags;

import java.util.*;

/**
 * A Select in an html document.
 *
 * @author Dan Howard
 */

public final class Select extends HTMLElement {

    private boolean multiple;

    // Not public because we don't want the user to access the arraylist directly.
    // The only way to add a new option is to use the addOption method.
    List<Option> options = new ArrayList<Option>(3);

    /**
     * Default constructor.
     */
    public Select() {
        tag = Tags.select;
        multiple = false;
    }


    /**
     * Construct a select with the specified name.
     *
     * @param name name of the select.
     */
    public Select(String name) {
        this();
        setName(name);
    }

    /**
     * This method adds a new option to the available options for the select object.
     *
     * @param option An option object to add to the select list
     * @return int - the new size of the option list.
     */
    public int addOption(Option option) {
        int index = -1;
        //option.parent = this;

        StringTokenizer st = new StringTokenizer(getValue(), Prefs.SelectValueSeperator.getValue());
        String itemValue;
        while (st.hasMoreTokens()) {
            itemValue = st.nextToken();
            if (itemValue.equals(option.getValue())) {
                option.setSelected(true);
            }
        }

        options.add(option);
        index = options.size();
        return index;
    }

    /**
     * Clears the list of options on the select.
     */
    public void clearOptions() {
        options.clear();
    }


    /**
     * Returns a clone of the select object.
     *
     * @return Select object.
     */
    @Override
    public Select clone() {
        Select select = (Select) super.clone();

        for (int j = 0; j < options.size(); j++) {
            select.addOption(new Option(options.get(j).getValue(), options.get(j).getText()));
        }
        return select;
    }

    /**
     * Returns the specified option object by the numeric index.
     *
     * @param index the index of the option to retrive
     * @return Option object.
     */
    public Option options(int index) {
        return options.get(index);
    }

    /**
     * Gets the value of the select object.
     *
     * @return value of the select object.
     */
    @Override
    public String getValue() {
        String selectValue = super.getValue();

        // Ensure that exisiting options are flagged properly.
        setValue(selectValue);

        String value = "";

        // Loop through the options and add the values of the 'selected' options
        // to the value string + commas...
        String sep = Prefs.SelectValueSeperator.getValue();
        for (int j = 0; j < options.size(); j++) {
            Option option = options.get(j);

            if (option.isSelected()) {
                value += option.getValue() + sep;
            }
        }

        // Strip last comma
        int n = value.lastIndexOf(",");
        if (n > -1) {
            value = value.substring(0, n);
        }

        if (value.trim().length() == 0 && selectValue.trim().length() > 0) {
            // If there was no value found in the options list but the user
            // had set a value in the select object itself then return that
            // value but log a warning.
            value = selectValue;
            log.warn("Returning Select object " + getName() + " assigned value: " + value + " which does not exist in the options list.");
        }
        return value;
    }

    /**
     * Indicates if the select is a multiple select.
     *
     * @return mulitple
     */
    public boolean isMultiple() {
        return multiple;
    }

    /**
     * Set the name of the select object.
     *
     * @param name name of the select object.
     */
    @Override
    public void setName(String name) {
        super.setName(name);
        setAttribute("name", name);
    }

    /**
     * Sets if this select is a multiple select.
     *
     * @param multiple boolean.
     */
    public void setMultiple(boolean multiple) {

        this.multiple = multiple;
        if (multiple) {
            setAttribute("multiple", "multiple");
        } else {
            removeAttribute("multiple");
        }
    }

    /**
     * Sets the value for the select object. The value to  assign
     * may be a single value or multiple values seperate by a comma.
     *
     * @param value String value to set.
     */
    @Override
    public void setValue(String value) {
        super.setValue(value);

        // locate the 1st option whose itemValue = value
        // If multi-select value could be comma seperated?
        for (int j = 0; j < options.size(); j++) {

            Option option = options.get(j);

            option.setSelected(false);

            StringTokenizer st = new StringTokenizer(value, Prefs.SelectValueSeperator.getValue());
            while (st.hasMoreTokens()) {
                String itemValue = st.nextToken();
                String optionValue = option.getValue();

                if (optionValue.equals(itemValue)) {
                    option.setSelected(true);
                    break;
                }
            }
        }

    }

    /**
     * Sort the options on the select.
     */
    public void sortOptions() {
        Collections.sort(options, new Comparator<Option>() {

            public int compare(Option o1, Option o2) {
                String s1 = o1.getText();
                String s2 = o2.getText();
                return s1.compareTo(s2);
            }
        });
    }


    /**
     * Renders the select.
     *
     * @return The select as a html string.
     */
    @Override
    public String toString() {

        if (swapWith != null) {
            return swapWith.toString();
        }

        if (!isRenderable() && !log.isDebugEnabled()) {
            return "";
        }

        // Used to ensure options are initialized properly.
        getValue();

        StringBuilder sb = new StringBuilder(super.toString());
        for (int j = 0; j < options.size(); j++) {
            sb.append(options.get(j).toString());
        }
        String ls = System.getProperty("line.separator");
        sb.append(ls);
        sb.append("</").append(tag).append(">");
        sb.append(ls);
        return sb.toString();
    }
}
