/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Aug 9, 2005
 * Time: 6:50:28 PM
 */
package net.sf.jolene.factories;

import net.sf.jolene.constants.Prefs;
import net.sf.jolene.dom.Document;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Map;


/**
 * DocumentFactory is a Singleton used to perform cacheing of documents. Documents added to the cache
 * are not parsed again. Instead they are cloned which is faster.
 *
 * @author Dan Howard
 */
public class DocumentFactory {

    private static final Logger log = LogManager.getLogger(DocumentFactory.class);

    private static final int DEFAULT_CACHE_SIZE = 32;

    private static DocumentFactory instance = new DocumentFactory();

    private Map<String, Document> documents;

    /**
     * Gets the factory instance.
     *
     * @return DocumentFactory instance.
     */
    public static DocumentFactory getInstance() {
        return instance;
    }

    private DocumentFactory() {
        int cacheSize;
        try {
            cacheSize = Integer.parseInt(Prefs.CacheSize.getValue());
        } catch (NumberFormatException e) {
            cacheSize = DEFAULT_CACHE_SIZE;
            log.warn("NumberFormatException : " + e.getMessage() + " cache size set to 32");
        }
        documents = new HashMap<String, Document>(cacheSize);
    }

    /**
     * Returns a new document instance using the full filename + timestamp as the factory key.
     *
     * @param docFileName full path and name to the file to get.
     * @return Document the document object from the cache.
     * @throws IOException if any IOException occurs.
     */
    public Document getDocument(String docFileName) throws IOException {
        log.debug("getDocument: " + docFileName);

        boolean cacheEnabled = Boolean.parseBoolean(Prefs.DocumentCaching.getValue());

        log.debug("getDocument: filename: Cache enabled? " + cacheEnabled);

        if (cacheEnabled) {
            File file = new File(docFileName);
            if (! file.exists()) {
                throw new FileNotFoundException(docFileName + " not found!");
            }
            String key = docFileName + " - " + file.lastModified();
            if (!documents.containsKey(key)) {
                documents.put(key, new Document(docFileName));
            }
            return documents.get(key).clone();
        }

        return new Document(docFileName);
    }

    /**
     * Returns a new document instance using the full filename + timestamp as the factory key.
     *
     * @param docFileName full path and name to the file to get.
     * @param contextPath aids in finding the document's real path from the uri.
     * @param uri         uri to the document.
     * @return Document   the document object from the cache.
     * @throws IOException if ever an IOException occurs.
     */
    public Document getDocument(String docFileName, String contextPath, String uri) throws IOException {
        log.debug("getDocument (with cp): " + docFileName + " cp: " + contextPath);

        boolean cacheEnabled = Boolean.parseBoolean(Prefs.DocumentCaching.getValue());
        log.debug("getDocument: filename, contextPath, uri: Cache enabled? " + cacheEnabled);

        if (cacheEnabled) {
            File file = new File(docFileName);
            if (! file.exists()) {
                throw new FileNotFoundException(docFileName + " not found!");
            }
            String key = docFileName + " - " + file.lastModified();
            if (!documents.containsKey(key)) {
                documents.put(key, new Document(docFileName, contextPath, uri));
            }
            return documents.get(key).clone();

        }
        return new Document(docFileName, contextPath, uri);
    }


}
