package net.sf.jolene.html;

import java.util.Collection;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * Class to manage a style string.
 *
 * @author Dan Howard
 * @since Aug 3, 2008 6:09:50 PM
 */
public final class Styles implements IStyles {

    private SortedMap<String, String> styles;


    /**
     * Default style constructor
     */
    public Styles() {
        styles = new TreeMap<String, String>();
    }


    /**
     * Construct a style object passing in formatted style string.
     *
     * @param styles formatted style string.
     */
    public Styles(String styles) {
        this();
        setStyles(styles);
    }

    /**
     * Get a style based on a style name.
     *
     * @param key case insensitive style name.
     * @return String value for the specified style.
     */
    public String getStyle(String key) {
        if (key == null) {
            return null;
        }
        return styles.get(key.toLowerCase());


    }

    /**
     * Indicates if a style exists.
     *
     * @param key case instensitive style key name.
     * @return true if the style exists in the string.
     */
    public boolean hasStyle(String key) {
        if (key == null) {
            return false;
        }
        return getStyle(key.toLowerCase()) != null;

    }


    /**
     * Returns a set of style keys from the Style.
     *
     * @return set of style keys from the Style.
     */
    public Set<String> keySet() {
        return styles.keySet();
    }

    /**
     * Removes a style from the Style object.
     *
     * @param key Style key name to remove.
     * @return previous value associated with specified key, or <tt>null</tt>
     *         if there was no mapping for key.
     */
    public String removeStyle(String key) {
        return styles.remove(key.toLowerCase());
    }


    /**
     * Clears the styles.
     */
    public void clear() {
        styles.clear();
    }

    /**
     * Set a style on the Style object.
     *
     * @param key   Style key name.
     * @param value Style value.
     * @return previous value associated with specified key, or <tt>null</tt> if there was no mapping for key.
     */
    public String setStyle(String key, String value) {

        // Don't allow null values
        if (value == null) {
            value = "";
        }

        return styles.put(key.toLowerCase(), value);
    }


    /**
     * Sets styles on the Style object based on a formatted style string.
     * Expects the string to be in style format.
     *
     * @param style string in a format like <tt>azimuth:behind;background:aliceblue;background-color:aquamarine;border-bottom-style:solid;clip:auto;border-top-width:medium;</tt>
     */
    public void setStyles(String style) {
        String astyles[] = style.split(";");
        for (int j = 0; j < astyles.length; j++) {
            String astyle[] = astyles[j].split(":");
            if (astyle.length == 2) {
                setStyle(astyle[0].trim(), astyle[1].trim());
            }
        }
    }

    /**
     * Sets the style based on the formatted style string, clearing all exisiting styles first.
     *
     * @param style string in a format like <tt>azimuth:behind;background:aliceblue;background-color:aquamarine;border-bottom-style:solid;clip:auto;border-top-width:medium;</tt>
     */
    public void resetStyles(String style) {
        clear();
        setStyles(style);
    }

    /**
     * Returns a Collection of Style values.
     *
     * @return Collection of Style values.
     */
    public Collection<String> values() {
        return styles.values();
    }


    /**
     * Renders this Style as a style string.
     *
     * @return a string formatted for a style attribute.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();

        Set<String> keys = keySet();
        for (String key : keys) {
            sb.append(key).append(":").append(getStyle(key)).append(";");
        }
        return sb.toString();
    }
}
